/*

*/

package binarytree;

/**
   This concrete subclass of {@link BTreeAbstract} defines a
   binary tree node. This is one case of the two cases in the
   algebraic data type for a binary tree.
*/
public class BTreeNode<T> extends BTreeAbstract<T>
{
   private T element;
   private BTreeAbstract<T> left;
   private BTreeAbstract<T> right;

   /**
      Construct a leaf node.

      @param element  reference to the data object to store in this node
      @throws NullPointerException if {@code element} is {@code null}
   */
   public BTreeNode(final T element)
   {
      this(element, new EmptyBTree<T>(), new EmptyBTree<T>());
   }


   /**
      Construct a {@code BTreeNode} with the given binary trees
      as its left and right branches.

      @param element  reference to the data object to store in this node
      @param left     left branch tree for this node
      @param right    right branch tree for this node
      @throws NullPointerException if {@code element} is {@code null}
      @throws NullPointerException if {@code left} is {@code null}
      @throws NullPointerException if {@code right} is {@code null}
   */
   public BTreeNode(final T element, final BTreeAbstract<T> left,
                                     final BTreeAbstract<T> right)
   {
      if (null == element)
         throw new NullPointerException("root element must not be null");
      if (null == left)
         throw new NullPointerException("left branch must not be null");
      if (null == right)
         throw new NullPointerException("right branch must not be null");

      this.element = element;
      this.left = left;
      this.right = right;
   }


   // Implement the four mthods of the BTree<T> interface.

   @Override
   public T root()
   {
      return element;
   }

   @Override
   public BTree<T> left()
   {
      return left;
   }

   @Override
   public BTree<T> right()
   {
      return right;
   }

   @Override
   public boolean isEmpty()
   {
      return false;
   }
}//BTreeNode
