/*

*/

package binarytree;

/**
   The {@code prettyPrint()} method takes a {@link BTree}
   and converts it into a well formatted {@link String}.
*/
public class PrettyPrinter
{
   public static final String INDENTATION = "  ";

   /**
      Pretty print a {@link BTree} with each sub tree
      indented by a certain amount.

      @param <T>   The element type for the {@link BTree}
      @param tree  {@link BTree} to be pretty printed
      @return the {@link String} representation of the prettyprinted {@link BTree}
   */
   public static <T> String prettyPrint(final BTree<T> tree)
   {
      return prettyPrint(tree, "");
   }


   /**
      This is a recursive helper function for the prettyPrint method.
      <p>
      This method pretty prints a {@link BTree} with a given amount
      of indentation. If the pretty printer needs to prettyprint a
      sub tree, then the sub tree gets a larger amount of indentation.
      <p>
      This method is essentially a pre-order traversal of the {@link BTree}.

      @param <T>     The element type for the {@link BTree}
      @param tree    {@link BTree} to be pretty printed
      @param indent  {@link String} holding the current left margin indentation
      @return the {@link String} representation of the given {@link BTree} prettyprinted and indented
   */
   private static <T> String prettyPrint(final BTree<T> tree,
                                         final String indent)
   {
      String result = indent;

      if ( tree.isEmpty() ) // stops the recursion
      {
         result += "()";
      }
      else if ( tree.isLeaf() ) // stops the recursion also
      {
         result += tree.root();
      }
      else
      {
         result += "(" + tree.root() + "\n";
         // recursively traverse the two sub trees
         result += prettyPrint( tree.left(),  indent+INDENTATION ) + "\n";
         result += prettyPrint( tree.right(), indent+INDENTATION ) + "\n";
         result += indent + ")";
      }

      return result;
   }
}
